"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const getStream = require("get-stream");
const path = require("path");
const exists = require("path-exists");
const pathTemp = require("path-temp");
const renameOverwrite = require("rename-overwrite");
const ssri = require("ssri");
const addFilesFromDir_1 = require("./addFilesFromDir");
const addFilesFromTarball_1 = require("./addFilesFromTarball");
const checkFilesIntegrity_1 = require("./checkFilesIntegrity");
exports.checkFilesIntegrity = checkFilesIntegrity_1.default;
const writeFile_1 = require("./writeFile");
function createCafs(cafsDir, ignore) {
    const locker = new Map();
    const addStream = addStreamToCafs.bind(null, locker, cafsDir);
    const addBuffer = addBufferToCafs.bind(null, locker, cafsDir);
    return {
        addFilesFromDir: addFilesFromDir_1.default.bind(null, { addBuffer, addStream }),
        addFilesFromTarball: addFilesFromTarball_1.default.bind(null, addStream, ignore !== null && ignore !== void 0 ? ignore : null),
    };
}
exports.default = createCafs;
async function addStreamToCafs(locker, cafsDir, fileStream) {
    const buffer = await getStream.buffer(fileStream);
    return addBufferToCafs(locker, cafsDir, buffer);
}
async function addBufferToCafs(locker, cafsDir, buffer) {
    const integrity = ssri.fromData(buffer);
    const fileDest = contentPathFromHex(cafsDir, integrity.hexDigest());
    if (locker.has(fileDest)) {
        await locker.get(fileDest);
        return integrity;
    }
    const p = (async () => {
        // This is a slow operation. Should be rewritten
        if (await exists(fileDest))
            return;
        // This might be too cautious.
        // The write is atomic, so in case pnpm crashes, no broken file
        // will be added to the store.
        // It might be a redundant step though, as we verify the contents of the
        // files before linking
        //
        // If we don't allow --no-verify-store-integrity then we probably can write
        // to the final file directly.
        const temp = pathTemp(path.dirname(fileDest));
        await writeFile_1.default(temp, buffer);
        await renameOverwrite(temp, fileDest);
    })();
    locker.set(fileDest, p);
    await p;
    return integrity;
}
function getFilePathInCafs(cafsDir, integrity) {
    return contentPathFromIntegrity(cafsDir, integrity);
}
exports.getFilePathInCafs = getFilePathInCafs;
function contentPathFromIntegrity(cafsDir, integrity) {
    const sri = ssri.parse(integrity, { single: true });
    return contentPathFromHex(cafsDir, sri.hexDigest());
}
function contentPathFromHex(cafsDir, hex) {
    return path.join(cafsDir, hex.slice(0, 2), hex.slice(2));
}
//# sourceMappingURL=index.js.map